#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai

__license__   = 'GPL v3'
__copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
__docformat__ = 'restructuredtext en'


from calibre.customize import InterfaceActionBase



class ActionSendByRSYNC(InterfaceActionBase):
  name                = 'Send by RSYNC'
  description         = 'Send book(s) by SSH/RSH'
  supported_platforms = ['linux', 'osx', 'windows'] # Platforms this plugin will run on
  author              = 'janvanmaar'
  version             = (1, 0, 2)

  default_site_customization = 'root@10.0.0.22:/mnt/base-us/documents/ ; rsync ; MOBI,EPUB,TXT ; 1'

  def __init__(self, path_to_plugin):
    InterfaceActionBase.__init__(self, path_to_plugin)
    from calibre.customize.ui import customize_plugin,plugin_customization
    if not plugin_customization(self):
      customize_plugin(self, self.default_site_customization)

  def customization_help(self, gui=1):
    # this is rather ugly; more fields input would be nicer...
    message = []
    message.append( "You can customize:  \n the remote address ; the rsync binary (and options) ; ")
    message.append( "the list of allowed book formats to sync ;  ")
    message.append( "whether to send Kindle file refresh command after rsync\n\n")
    message.append( "Example (using the default values:\n" )
    message.append( "root@10.0.0.22:/mnt/base-us/documents/ ; rsync ; MOBI,EPUB,TXT ; 1\n\n" )
    message.append( "You might need to restart Calibre for the changes to take effect." )
    return ''.join(message)


  def load_actual_plugin(self, gui):
    import subprocess,os
    #from PyQt4.Qt import QMenu, QToolButton
    from calibre.gui2.actions import InterfaceAction
    from calibre.gui2 import error_dialog,info_dialog


    class SendByRSYNCAction(InterfaceAction):
      name = 'Send by RSYNC'
      action_spec = (_('Send by RSYNC'), None, None, _('Ctrl+Shift+S'))
      action_type = 'current'

      rsync_path='rsync'
      formats=('MOBI','EPUB','TXT')
      kindle_refresh=1
      remote_host='root@10.0.0.22'
      remote_dir='/mnt/base-us/documents/'

      def __init__(self, gui, site_customization):
        InterfaceAction.__init__(self, gui, site_customization)
        # parse the user's customized input
        if site_customization:
          user_args = site_customization.split(';')
          user_args = [ usr_arg.strip()  for usr_arg in user_args ]
          if user_args:
            spl = user_args[0].split(':')
            if len(spl)>2:
              return error_dialog(self.gui, _('Wrong customization'), _('Too many colons (:) in remote host: {0}'.format(user_args)), show=True)
            else:
              self.remote_host=spl[0]
              self.remote_dir=''
              if len(spl)>1:
                self.remote_dir=spl[1]
            try:
              self.rsync_path=user_args[1]
              self.formats=user_args[2].split(',')
              self.kindle_refresh=user_args[3]
            except:
              pass


      def genesis(self):
        self.qaction.triggered.connect(self.send_by_rsync)

      def send_by_rsync(self,  ):
        print 'debug -2'
        row = self.gui.library_view.currentIndex()
        if not row.isValid():
          return error_dialog(self.gui, _('Cannot rsync'), _('No book selected.'), show=True)
        # Find suitable format
        book_id = self.gui.library_view.model().id(row)
        path_to_file = None
        print 'debug -1'
        import sys
        sys.stdout.flush()
        for format in self.formats:
          try:
            print 'debug0',path_to_file
            path_to_file = self.gui.library_view.model().db.format_abspath(
                           book_id, format, index_is_id=True)
            print 'debug',path_to_file
          except:
            pass
          else:
            if path_to_file is not None:
              break
        else:
          return error_dialog(self.gui, _('Cannot rsync'),
                    _('No suitable format of book available, convert first.'), show=True)
        # Confirm we have defined the path etc
        if self.rsync_path is None:
          return error_dialog(self.gui, _('rsync binary not defined'),
                             _('Specify a value for epub_editor_path in current tweaks.'))
        proc=subprocess.Popen( self.rsync_path +' "'+ path_to_file + '" {0}:{1}'.format(
                               self.remote_host,self.remote_dir), stderr=subprocess.PIPE, shell=True )
        out,err=proc.communicate()
        # this is a pretty error-prone heuristic. However, according to the rsync documentation, 
        # it is a feature that rsync/ssh returns *any* messages from the remote host to stderr....
        if err and not 'welcome' in err.lower() and \
           any(1 for k in ('error','problem','unable','refused','timeout','denied','bad ') if k in err.lower()):
          return error_dialog(self.gui, _('Error rsyncing:'), _(err), show=True)
        else:
          if self.kindle_refresh:
            proc=subprocess.Popen( ['ssh', '{0}'.format(self.remote_host),
                     'dbus-send --system /default com.lab126.powerd.resuming int32:1'], stderr=subprocess.PIPE )
#            proc=subprocess.Popen( ['ssh', '{0}'.format(self.remote_host), '-p 2222', 'ls'], stderr=subprocess.PIPE )
            out,err=proc.communicate()
            if err and not 'welcome' in err.lower() and \
               any(1 for k in ('error','problem','unable','refused','timeout','denied','bad ') if k in err.lower()):
              return error_dialog(self.gui, _('Error refreshing files on Kindle:'), _(err), show=True)
          book=os.path.basename(path_to_file)
          return info_dialog(self.gui, _('Successfully rsynced'),
               _('Rsync of {0} successfull.'.format(book)), show=True)

    #end of SendByRSYNCAction class

    return SendByRSYNCAction(gui, self.site_customization)
